diff -urN dnsmasq-2.45.orig/dbus/DBus-interface dnsmasq-2.45/dbus/DBus-interface
--- dnsmasq-2.45.orig/dbus/DBus-interface	2008-07-20 21:18:16.000000000 +0200
+++ dnsmasq-2.45/dbus/DBus-interface	2008-09-15 11:55:22.000000000 +0200
@@ -91,4 +91,18 @@
 specified by via the DBus, but it leaves any servers specified via the
 command line or /etc/dnsmasq.conf or /etc/resolv.conf alone.
 
+Available signals are:
+
+DhcpPacket
+----------
+
+This signal is emitted whenever a DHCP packet is received or sent.
+
+Example:
+
+STRING "DHCPREQUEST"
+STRING "eth0"
+STRING "192.168.1.115"
+STRING "01:23:45:67:89:ab"
+STRING "message"
 
diff -urN dnsmasq-2.45.orig/src/dbus.c dnsmasq-2.45/src/dbus.c
--- dnsmasq-2.45.orig/src/dbus.c	2008-07-20 21:18:16.000000000 +0200
+++ dnsmasq-2.45/src/dbus.c	2008-09-15 11:58:03.000000000 +0200
@@ -352,4 +352,47 @@
     }
 }
 
+void emit_dbus_dhcp_packet(char *type, void *addr, unsigned char *ext_mac, 
+			   int mac_len, char *interface, char *string)
+{
+  struct in_addr a;
+  DBusConnection *connection = (DBusConnection *)daemon->dbus;
+  DBusMessage* message;
+  DBusMessageIter args;
+  char messagetype[16];
+  const char *value;
+
+  if (!connection)
+    return;
+
+  if (!(message = dbus_message_new_signal(DNSMASQ_PATH, DNSMASQ_SERVICE, "DhcpPacket")))
+    return;
+
+  /* addr may be misaligned */
+  if (addr)
+    memcpy(&a, addr, sizeof(a));
+ 
+  dbus_message_iter_init_append(message, &args);
+
+  snprintf(messagetype, sizeof(messagetype), "%s%s",
+	   type ? "DHCP" : "BOOTP",
+	   type ? type : "");
+  value = messagetype;
+  dbus_message_iter_append_basic(&args, DBUS_TYPE_STRING, &value);
+
+  dbus_message_iter_append_basic(&args, DBUS_TYPE_STRING, &interface);
+
+  value = addr ? inet_ntoa(a) : "";
+  dbus_message_iter_append_basic(&args, DBUS_TYPE_STRING, &value);
+
+  value = print_mac(daemon->namebuff, ext_mac, mac_len);
+  dbus_message_iter_append_basic(&args, DBUS_TYPE_STRING, &value);
+
+  value = string ? string : "";
+  dbus_message_iter_append_basic(&args, DBUS_TYPE_STRING, &value);
+
+  dbus_connection_send(connection, message, NULL);
+  dbus_message_unref(message);
+}
+
 #endif
diff -urN dnsmasq-2.45.orig/src/dnsmasq.h dnsmasq-2.45/src/dnsmasq.h
--- dnsmasq-2.45.orig/src/dnsmasq.h	2008-07-20 21:18:16.000000000 +0200
+++ dnsmasq-2.45/src/dnsmasq.h	2008-09-15 11:32:36.000000000 +0200
@@ -827,6 +827,8 @@
 char *dbus_init(void);
 void check_dbus_listeners(fd_set *rset, fd_set *wset, fd_set *eset);
 void set_dbus_listeners(int *maxfdp, fd_set *rset, fd_set *wset, fd_set *eset);
+void emit_dbus_dhcp_packet(char *type, void *addr, unsigned char *ext_mac, 
+			   int mac_len, char *interface, char *string);
 #endif
 
 /* helper.c */
diff -urN dnsmasq-2.45.orig/src/rfc2131.c dnsmasq-2.45/src/rfc2131.c
--- dnsmasq-2.45.orig/src/rfc2131.c	2008-07-20 21:18:16.000000000 +0200
+++ dnsmasq-2.45/src/rfc2131.c	2008-09-15 11:33:06.000000000 +0200
@@ -1172,6 +1172,10 @@
 	    addr ? " " : "",
 	    print_mac(daemon->namebuff, ext_mac, mac_len),
 	    string ? string : "");
+
+#ifdef HAVE_DBUS
+  emit_dbus_dhcp_packet(type, addr, ext_mac, mac_len, interface, string);
+#endif
 }
 
 static void log_options(unsigned char *start)
